# ################################################################# #
#### LOAD LIBRARY AND DEFINE CORE SETTINGS                       ####
# ################################################################# #

### Clear memory
rm(list = ls())

### Load Apollo library
library(apollo)

### Initialise code
apollo_initialise()

### Set core controls
apollo_control = list(
  modelName       = "RRM",
  modelDescr      = "Simple RRM model on mode choice SP data",
  indivID         = "ID", 
  outputDirectory = "output"
)

# ################################################################# #
#### LOAD DATA AND APPLY ANY TRANSFORMATIONS                     ####
# ################################################################# #

### Loading data from package
### if data is to be loaded from a file (e.g. called data.csv), 
### the code would be: database = read.csv("data.csv",header=TRUE)
database = apollo_modeChoiceData
### for data dictionary, use ?apollo_modeChoiceData

### Use only SP data
database = subset(database,database$SP==1)

### Create new variable with average income
database$mean_income = mean(database$income)

# ################################################################# #
#### DEFINE MODEL PARAMETERS                                     ####
# ################################################################# #

### Vector of parameters, including any that are kept fixed in estimation
apollo_beta=c(asc_car     = 0,
              asc_bus     = 0,
              asc_air     = 0,
              asc_rail    = 0,
              b_tt_car    = 0,
              b_tt_bus    = 0,
              b_tt_air    = 0,
              b_tt_rail   = 0,
              b_access    = 0,
              b_cost      = 0,
              b_no_frills = 0,
              b_wifi      = 0,
              b_food      = 0,
              mu_rrm      = 1)

### Vector with names (in quotes) of parameters to be kept fixed at their starting value in apollo_beta, use apollo_beta_fixed = c() if none
apollo_fixed = c("asc_car","b_no_frills","mu_rrm")

# ################################################################# #
#### GROUP AND VALIDATE INPUTS                                   ####
# ################################################################# #

apollo_inputs = apollo_validateInputs()

# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){
    
  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))

  ### Create list of probabilities P
  P = list()
  
  ### Create RRM settings
  rrm_settings=list(
    alternatives  = c(car=1, bus=2, air=3, rail=4),
    avail         = list(car=av_car, bus=av_bus, air=av_air, rail=av_rail),
    choiceVar     = choice,
    rum_inputs    = list(car  = asc_car,
                         bus  = asc_bus,
                         air  = asc_air, 
                         rail = asc_rail), 
    regret_inputs = list(
      time=list(x=list(time_car, time_bus, time_air, time_rail),
                b=list(b_tt_car,b_tt_bus,b_tt_air,b_tt_rail)),
      cost=list(x=list(cost_car, cost_bus, cost_air, cost_rail),
                b=list(b_cost)),
      access=list(x=list(0, access_bus, access_air, access_rail),
                  b=list(b_access)),
      frills=list(x=list(0,
                      0,
                      b_no_frills * ( service_air  == 1 ) + b_wifi * ( service_air  == 2 ) + b_food * ( service_air  == 3 ),
                      b_no_frills * ( service_rail == 1 ) + b_wifi * ( service_rail == 2 ) + b_food * ( service_rail == 3 )),
                  b=1)),
    regret_scale  = list(mu_rrm)
  )

  ### Compute probabilities using RRM model
  P[["model"]] = apollo_rrm(rrm_settings, functionality)
  
  ### Take product across observation for same individual
  P = apollo_panelProd(P, apollo_inputs, functionality)
  
  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# ################################################################# #
#### MODEL ESTIMATION                                            ####
# ################################################################# #

model = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)

# ################################################################# #
#### MODEL OUTPUTS                                               ####
# ################################################################# #

# ----------------------------------------------------------------- #
#---- FORMATTED OUTPUT (TO SCREEN)                               ----
# ----------------------------------------------------------------- #

apollo_modelOutput(model)

# ----------------------------------------------------------------- #
#---- FORMATTED OUTPUT (TO FILE, using model name)               ----
# ----------------------------------------------------------------- #

apollo_saveOutput(model)

# ################################################################# #
##### POST-PROCESSING                                            ####
# ################################################################# #

### Print outputs of additional diagnostics to new output file (remember to close file writing when complete)
apollo_sink()

# ----------------------------------------------------------------- #
#---- Ben-Akiva & Swait tests for non-nested models              ----
# ----------------------------------------------------------------- #

apollo_basTest(model,"../1 MNL/output/MNL_SP")

# ----------------------------------------------------------------- #
#---- switch off writing to file                                 ----
# ----------------------------------------------------------------- #

apollo_sink()
