# ################################################################# #
#### LOAD LIBRARY AND DEFINE CORE SETTINGS                       ####
# ################################################################# #

### Clear memory
rm(list = ls())

### Load Apollo library
library(apollo)

### Initialise code
apollo_initialise()

### Set core controls
apollo_control = list(
  modelName       = "BW_joint_model_diff_params",
  modelDescr      = "Best-worst model on drug choice data, using different parameters for best and worst choices",
  indivID         = "ID", 
  outputDirectory = "output"
)

# ################################################################# #
#### LOAD DATA AND APPLY ANY TRANSFORMATIONS                     ####
# ################################################################# #

### Loading data from package
### if data is to be loaded from a file (e.g. called data.csv), 
### the code would be: database = read.csv("data.csv",header=TRUE)
database = apollo_drugChoiceData
### for data dictionary, use ?apollo_drugChoiceData

# ################################################################# #
#### DEFINE MODEL PARAMETERS                                     ####
# ################################################################# #

### Vector of parameters, including any that are kept fixed in estimation
apollo_beta = c(b_brand_Artemis_best     = 0, b_brand_Artemis_worst     = 0, 
                b_brand_Novum_best       = 0, b_brand_Novum_worst       = 0, 
                b_brand_BestValue_best   = 0, b_brand_worstValue_worst  = 0, 
                b_brand_Supermarket_best = 0, b_brand_Supermarket_worst = 0, 
                b_brand_PainAway_best    = 0, b_brand_PainAway_worst    = 0, 
                b_country_CH_best        = 0, b_country_CH_worst        = 0, 
                b_country_DK_best        = 0, b_country_DK_worst        = 0, 
                b_country_USA_best       = 0, b_country_USA_worst       = 0, 
                b_country_IND_best       = 0, b_country_IND_worst       = 0, 
                b_country_RUS_best       = 0, b_country_RUS_worst       = 0, 
                b_country_BRA_best       = 0, b_country_BRA_worst       = 0, 
                b_char_standard_best     = 0, b_char_standard_worst     = 0, 
                b_char_fast_best         = 0, b_char_fast_worst         = 0, 
                b_char_double_best       = 0, b_char_double_worst       = 0, 
                b_risk_best              = 0, b_risk_worst              = 0, 
                b_price_best             = 0, b_price_worst             = 0)

### Vector with names (in quotes) of parameters to be kept fixed at their starting value in apollo_beta, use apollo_beta_fixed = c() if none
apollo_fixed = c("b_brand_Artemis_best", "b_country_USA_best", "b_char_standard_best",
                 "b_brand_Artemis_worst", "b_country_USA_worst", "b_char_standard_worst")

# ################################################################# #
#### GROUP AND VALIDATE INPUTS                                   ####
# ################################################################# #

apollo_inputs = apollo_validateInputs()

# ################################################################# #
#### DEFINE MODEL AND LIKELIHOOD FUNCTION                        ####
# ################################################################# #

apollo_probabilities=function(apollo_beta, apollo_inputs, functionality="estimate"){

  ### Attach inputs and detach after function exit
  apollo_attach(apollo_beta, apollo_inputs)
  on.exit(apollo_detach(apollo_beta, apollo_inputs))

  ### Create list of probabilities P
  P = list()
  
  ### List of utilities for the "best" choice
  V_best = list()
  V_best[["alt1"]] = ( b_brand_Artemis_best*(brand_1=="Artemis") + b_brand_Novum_best*(brand_1=="Novum") 
                       + b_country_CH_best*(country_1=="Switzerland") + b_country_DK_best*(country_1=="Denmark") + b_country_USA_best*(country_1=="USA") 
                       + b_char_standard_best*(char_1=="standard") + b_char_fast_best*(char_1=="fast acting") + b_char_double_best*(char_1=="double strength") 
                       + b_risk_best*side_effects_1
                       + b_price_best*price_1)
  V_best[["alt2"]] = ( b_brand_Artemis_best*(brand_2=="Artemis") + b_brand_Novum_best*(brand_2=="Novum") 
                       + b_country_CH_best*(country_2=="Switzerland") + b_country_DK_best*(country_2=="Denmark") + b_country_USA_best*(country_2=="USA") 
                       + b_char_standard_best*(char_2=="standard") + b_char_fast_best*(char_2=="fast acting") + b_char_double_best*(char_2=="double strength") 
                       + b_risk_best*side_effects_2
                       + b_price_best*price_2)
  V_best[["alt3"]] = ( b_brand_BestValue_best*(brand_3=="BestValue") + b_brand_Supermarket_best*(brand_3=="Supermarket") + b_brand_PainAway_best*(brand_3=="PainAway") 
                       + b_country_USA_best*(country_3=="USA") + b_country_IND_best*(country_3=="India") + b_country_RUS_best*(country_3=="Russia") + b_country_BRA_best*(country_3=="Brazil") 
                       + b_char_standard_best*(char_3=="standard") + b_char_fast_best*(char_3=="fast acting") 
                       + b_risk_best*side_effects_3
                       + b_price_best*price_3 )
  V_best[["alt4"]] = ( b_brand_BestValue_best*(brand_4=="BestValue") + b_brand_Supermarket_best*(brand_4=="Supermarket") + b_brand_PainAway_best*(brand_4=="PainAway") 
                       + b_country_USA_best*(country_4=="USA") + b_country_IND_best*(country_4=="India") + b_country_RUS_best*(country_4=="Russia") + b_country_BRA_best*(country_4=="Brazil") 
                       + b_char_standard_best*(char_4=="standard") + b_char_fast_best*(char_4=="fast acting") 
                       + b_risk_best*side_effects_4
                       + b_price_best*price_4 )

  ### Compute probabilities for "best" choice using MNL model
  mnl_settings_best = list(
    alternatives  = c(alt1=1, alt2=2, alt3=3, alt4=4),
    avail         = list(alt1=1, alt2=1, alt3=1, alt4=1),
    choiceVar     = best,
    utilities     = V_best,
    componentName = "best"
  )
  P[["choice_best"]] = apollo_mnl(mnl_settings_best, functionality)
  
  ### List of utilities for the "worse" choice
  V_worst = list()
  V_worst[["alt1"]] = ( b_brand_Artemis_worst*(brand_1=="Artemis") + b_brand_Novum_worst*(brand_1=="Novum") 
                        + b_country_CH_worst*(country_1=="Switzerland") + b_country_DK_worst*(country_1=="Denmark") + b_country_USA_worst*(country_1=="USA") 
                        + b_char_standard_worst*(char_1=="standard") + b_char_fast_worst*(char_1=="fast acting") + b_char_double_worst*(char_1=="double strength") 
                        + b_risk_worst*side_effects_1
                        + b_price_worst*price_1)
  V_worst[["alt2"]] = ( b_brand_Artemis_worst*(brand_2=="Artemis") + b_brand_Novum_worst*(brand_2=="Novum") 
                        + b_country_CH_worst*(country_2=="Switzerland") + b_country_DK_worst*(country_2=="Denmark") + b_country_USA_worst*(country_2=="USA") 
                        + b_char_standard_worst*(char_2=="standard") + b_char_fast_worst*(char_2=="fast acting") + b_char_double_worst*(char_2=="double strength") 
                        + b_risk_worst*side_effects_2
                        + b_price_worst*price_2)
  V_worst[["alt3"]] = ( b_brand_worstValue_worst*(brand_3=="BestValue") + b_brand_Supermarket_worst*(brand_3=="Supermarket") + b_brand_PainAway_worst*(brand_3=="PainAway") 
                        + b_country_USA_worst*(country_3=="USA") + b_country_IND_worst*(country_3=="India") + b_country_RUS_worst*(country_3=="Russia") + b_country_BRA_worst*(country_3=="Brazil") 
                        + b_char_standard_worst*(char_3=="standard") + b_char_fast_worst*(char_3=="fast acting") 
                        + b_risk_worst*side_effects_3
                        + b_price_worst*price_3 )
  V_worst[["alt4"]] = ( b_brand_worstValue_worst*(brand_4=="BestValue") + b_brand_Supermarket_worst*(brand_4=="Supermarket") + b_brand_PainAway_worst*(brand_4=="PainAway") 
                        + b_country_USA_worst*(country_4=="USA") + b_country_IND_worst*(country_4=="India") + b_country_RUS_worst*(country_4=="Russia") + b_country_BRA_worst*(country_4=="Brazil") 
                        + b_char_standard_worst*(char_4=="standard") + b_char_fast_worst*(char_4=="fast acting") 
                        + b_risk_worst*side_effects_4
                        + b_price_worst*price_4 )
  
  ### Compute probabilities for "worst" choice using MNL model
  mnl_settings_worst = list(
    alternatives  = c(alt1=1, alt2=2, alt3=3, alt4=4),
    avail         = list(alt1=(best!=1), alt2=(best!=2), alt3=(best!=3), alt4=(best!=4)),
    choiceVar     = worst, 
    utilities     = list(alt1 = -V_worst[["alt1"]],
                         alt2 = -V_worst[["alt2"]],
                         alt3 = -V_worst[["alt3"]],
                         alt4 = -V_worst[["alt4"]])
  )
  P[["choice_worst"]] = apollo_mnl(mnl_settings_worst, functionality)

  ### Combined model
  P = apollo_combineModels(P, apollo_inputs, functionality)

  ### Take product across observation for same individual
  P = apollo_panelProd(P, apollo_inputs, functionality)

  ### Prepare and return outputs of function
  P = apollo_prepareProb(P, apollo_inputs, functionality)
  return(P)
}

# ################################################################# #
#### MODEL ESTIMATION                                            ####
# ################################################################# #

model = apollo_estimate(apollo_beta, apollo_fixed, apollo_probabilities, apollo_inputs)

# ################################################################# #
#### MODEL OUTPUTS                                               ####
# ################################################################# #

# ----------------------------------------------------------------- #
#---- FORMATTED OUTPUT (TO SCREEN)                               ----
# ----------------------------------------------------------------- #

apollo_modelOutput(model)

# ----------------------------------------------------------------- #
#---- FORMATTED OUTPUT (TO FILE, using model name)               ----
# ----------------------------------------------------------------- #

apollo_saveOutput(model)

